import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_cloud_messaging_bloc/src/core/exceptions/notification_exeption.dart';
import 'package:wyatt_cloud_messaging_bloc/src/domain/data_sources/remote/notification_remote_data_source.dart';
import 'package:wyatt_cloud_messaging_bloc/src/domain/entities/remote_notifications.dart';
import 'package:wyatt_cloud_messaging_bloc/src/domain/repositories/notification_repository.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

class NotificationRepositoryImpl extends NotificationRepository {
  final NotificationRemoteDataSource _notificationRemoteDataSource;

  NotificationRepositoryImpl({
    required NotificationRemoteDataSource notificationRemoteDataSource,
  }) : _notificationRemoteDataSource = notificationRemoteDataSource;
  @override
  FutureOrResult<void> register() => Result.tryCatchAsync(
        _notificationRemoteDataSource.register,
        (error) => error is AppException
            ? error
            : NotificationException(error.toString()),
      );

  @override
  FutureOrResult<String> getToken() => Result.tryCatchAsync(
        _notificationRemoteDataSource.getToken,
        (error) => error is AppException
            ? error
            : NotificationException(error.toString()),
      );

  @override
  FutureOrResult<void> subscribeToTopic(String topic) => Result.tryCatchAsync(
        () => _notificationRemoteDataSource.subscribeToTopic(topic),
        (error) => error is AppException
            ? error
            : NotificationException(error.toString()),
      );

  @override
  FutureOrResult<void> unsubscribeFromTopic(String topic) =>
      Result.tryCatchAsync(
        () => _notificationRemoteDataSource.unsubscribeFromTopic(topic),
        (error) => error is AppException
            ? error
            : NotificationException(error.toString()),
      );

  @override
  FutureOrResult<void> requestPermissions() => Result.tryCatchAsync(
        _notificationRemoteDataSource.requestPermissions,
        (error) => error is AppException
            ? error
            : NotificationException(error.toString()),
      );

  @override
  FutureOrResult<Stream<RemoteNotification>> onNotificationAccepted() =>
      Result.tryCatchAsync(
        _notificationRemoteDataSource.onNotificationBackgroundAccepted,
        (error) => error is AppException
            ? error
            : NotificationException(error.toString()),
      );

  @override
  FutureOrResult<Stream<RemoteNotification>>
      onNotificationBackgroundAccepted() => Result.tryCatchAsync(
            _notificationRemoteDataSource.onNotificationBackgroundAccepted,
            (error) => error is AppException
                ? error
                : NotificationException(error.toString()),
          );

  @override
  FutureOrResult<Stream<RemoteNotification>>
      onNotificationForegroundAccepted() => Result.tryCatchAsync(
            _notificationRemoteDataSource.onNotificationForegroundAccepted,
            (error) => error is AppException
                ? error
                : NotificationException(error.toString()),
          );
}
