import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_cloud_messaging_bloc/src/domain/entities/remote_notifications.dart';
import 'package:wyatt_cloud_messaging_bloc/src/domain/usecases/listen_notification_use_case.dart';
import 'package:wyatt_cloud_messaging_bloc/src/domain/usecases/subscribe.dart';
import 'package:wyatt_cloud_messaging_bloc/src/domain/usecases/unsubscribe.dart';

part 'cloud_messaging_state.dart';

typedef OnNotification = void Function(RemoteNotification);

class CloudmessagingCubit extends Cubit<CloudmessagingState> {
  final ListenNotification _listenNotification;
  final Subscribe _subscribe;
  final Unsubscribe _unsubscribe;

  final OnNotification? handleNotification;
  CloudmessagingCubit({
    required ListenNotification listenNotification,
    required Subscribe subscribe,
    required Unsubscribe unsubscribe,
    this.handleNotification,
  })  : _listenNotification = listenNotification,
        _subscribe = subscribe,
        _unsubscribe = unsubscribe,
        super(const CloudmessagingState());

  Future<void> listenNotification() async {
    final notificaitons = await _listenNotification.execute(const NoParam());
    if (notificaitons.isOk) {
      notificaitons.ok?.listen((notification) {
        emit(CloudmessagingState(notification));
      });
    }
  }

  FutureOr<void> subscribe(String topic) => _subscribe.execute(topic);

  FutureOr<void> unsubscribe(String topic) => _unsubscribe.execute(topic);
}
